﻿################################################################################
########################## Error Handling 1 ####################################
################################################################################
################################################################################
################################################################################

<#
Things don't always go as planned.  And when they don't you need to be able to handle
the errors and do something constructive with them.  Errors are also called exceptions,
and whenever you let an error come up to the user, it's called an unhandled exception.
If at all possible, this should never happen.
You should always handle your errors.
#>

## We try to stop a process that doesn't exist.  It shoots up an error.
stop-process 333333
## Now we can just suppress the error.
stop-process 333333 -erroraction silentlycontinue

## Now we can force a stop on the error.
stop-process 333333 -erroraction silentlycontinue

## Now we can just suppress the error.
# Here you give the variable a name, but you don't don't use the $ in the syntax.
# After you name the variable and fill it, you can use it.
stop-process 333333 -erroraction silentlycontinue -errorvariable err
$err

## If you have 2 cmds you'll overwrite the var each time.
stop-process 333333 -erroraction silentlycontinue -errorvariable err
$err.exception
stop-process 444444 -erroraction silentlycontinue -errorvariable err
$err.exception

## But if you put a + in front of the var it adds to the current error.
stop-process 333333 -erroraction silentlycontinue -errorvariable +err
stop-process 444444 -erroraction silentlycontinue -errorvariable +err
$err.exception


## Now we can get into some conditional logic to do something with the error if it exists.
dir z: -ErrorVariable Err -ErrorAction SilentlyContinue
if ($Err) { write-host "An error happend!!!" }

## Now let's look at another situation.  We're going to get a service that isn't there.
## It's not always that easy though.  It doesn't handle all types of errors.
Get-Service "Today" -ErrorAction SilentlyContinue -ErrorVariable err
if ($Err) { write-host "An error happend!!!" }

## But if you pipe it to out-file, it doesn't capture the error.
Get-Service | Out-File K:\Services.txt -ErrorAction SilentlyContinue -ErrorVariable err
##So there's a real reason why this doesn't work.  The file location doesn't exist.
##This introduces what's called a terminating error.  That's just what it sounds like.
##The error is so bad that the cmdlet can't continue... it terminates.
##The ErrorAction only affects non-terminating errors.


##First, lets see how we can catch the error, then we'll have a discussion about
##the nuances.
Try { Get-Service | Out-File K:\Services.txt }
Catch { write-host "This is an error!!!!" }

##This error is different in that it's non-terminating.  
##The file doesn't exist, so its contents can't be read, but that's nothing that 
##will keep it from trying to read the file.
##In the above error, you're trying to write to a location that doesn't exist.
##So this terminates the cmdlet in a very abrupt way.
##It's kinda like being told to go get something out of a bucket.
##You can go look in the bucket and there either will or won't be anything in it.
##But if you went one time and the bucket were missing, now that's a completely
##different issue completely.
##So here's a non-terminating error.  We need to look at different ErrorActions.
Get-Content K:\MyFile.txt -ErrorAction SilentlyContinue;

##Above I set the ErrorAction to SilentlyContinue.  This tells PS to just shutup and
##go to the next line in the script.
Get-Content K:\MyFile.txt -ErrorAction Stop;
"Next line."
Get-Content K:\MyFile.txt -ErrorAction Continue;
"Next line."
Get-Content K:\MyFile.txt -ErrorAction Ignore;
"Next line."
Get-Content K:\MyFile.txt -ErrorAction Inquire;
"Next line."

##The above are examples of setting ErrorAction at the stmt level.
##However, you can set it at the session level as well.
$ErrorActionPreference = "SilentlyContinue";
Get-Content K:\MyFile.txt

$ErrorActionPreference = "Stop";
Get-Content K:\MyFile.txt

##Now let's handle the error so we can do something constructive.
$ErrorActionPreference = "SilentlyContinue";
Get-Content K:\MyFile.txt -ErrorVariable err;
$err ##Just printing the error here.
##The error above is officially handled, even though we're just printing it.

##We can do better though.  Let's say something better.
$ErrorActionPreference = "SilentlyContinue";
Get-Content K:\MyFile.txt -ErrorVariable err;
If ($err -ne $null) {"We had an error."}

##What info is available for us in $err though?  Anybody know how to get that info?








$err | gm;

##Let's play with $err options now.
$ErrorActionPreference = "SilentlyContinue";
Get-Content K:\MyFile.txt -ErrorVariable err;
$err.Exception;
$err | gm;

$ErrorActionPreference = "SilentlyContinue";
Get-Content K:\MyFile.txt -ErrorVariable err;
$err.CategoryInfo;
$err | gm;

$ErrorActionPreference = "SilentlyContinue";
Get-Content K:\MyFile.txt -ErrorVariable err;
$err.CategoryInfo.Reason;
$err | gm;



################# Exercises #################3

<#
1. Try to read the contents of a file that doesn't exist, K:\SeansGreat.txt.
	When you get the error, handle it so it doesn't just spill red to the screen.
	Remember there are 2 methods.  Use the ErrorAction method this time.
2. Try to divide 1 by 0. 1/0.  Then handle the error so that it prints a message
	to the screen.
#>














